/***********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmp_PrinterSensor.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Xpm-PrintMonPlugin)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the status info structure entry in the registry
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Xpm_Pmp_System.h"
#include "Xpm_PrintMonPlugin.h"
#include "Xpm_Pmp_Interface.h"
#include "Xpm_Pmp_RegistryAccess.h"
#include "Xpm_Pmp_PrinterSensor.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Pse_InitializePrinterSensor
// ABSTRACT   Initializes printer information 
//******************************************************************************************

bool Xpm_Pmp_Pse_InitializePrinterSensor
   (P_XPM_PLUGIN_DATA psData)
{
   bool                      fResult        = false;  // Return value, FALSE by default
   TS_XPM_PMP_PRINTER_SENSOR sPrinterSensor;          // Printer sensor data structure (registry)
   
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize printer inquiry data structure 
   memset(&sPrinterSensor, 0, sizeof(sPrinterSensor));       // Clear complete structure
   sPrinterSensor.dwSize          = sizeof(sPrinterSensor);  // Set size of this structure
   sPrinterSensor.ubRevision      = D_XPM_PMP_PSE_REVISION;  // Set revision number (D_XPM_PMP_PSE_REVISION)
   sPrinterSensor.dwTsLastUpdate  = 0;                       // Set timestamp of last update
   sPrinterSensor.dwDataValidFlag = 0;                       // Reset data valid flags (see D_XPM_PMP_PSE_DVF_...)

   //-- Set time stamp
   sPrinterSensor.dwTsLastUpdate = GetTickCount();

   //-- Write status info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_PRINTER_SENSOR, (PBYTE) &sPrinterSensor, sizeof(sPrinterSensor)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Pse_UpdatePrinterSensor
// ABSTRACT   Updates the status info 
//******************************************************************************************

bool Xpm_Pmp_Pse_UpdatePrinterSensor
   (P_XPM_PLUGIN_DATA psData, bool fPrinterMightBeBusy, bool fInvalidateOnly)
{
   bool                            fResult                    = false;                  // Return value, FALSE by default
   TS_XPM_PMP_PRINTER_SENSOR       sPrinterSensor;                                      // Printer sensor data structure (registry)
   BYTE                            aubEcPrinterSensor[]       = {0x1d,0x61,0x00,0x04};  // GS a+[n1]+[n2],n=0x0004 => emulation command for requesting printer sensor
   BYTE                            aubPrinterSensor[sizeof(TS_XPM_API_PCKT_HEADER) + sizeof(TS_XPM_API_PCKT_PRINTER_SENSOR)]; 
                                                                                        // Printer status bytes
   DWORD                           dwReadTimeOutMs;                                     // Read timeout in ms
   
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize status info structure 
   memset(&sPrinterSensor, 0, sizeof(sPrinterSensor));       // Clear complete structure
   sPrinterSensor.dwSize          = sizeof(sPrinterSensor);  // Set size of this structure
   sPrinterSensor.ubRevision      = D_XPM_PMP_PSE_REVISION;  // Set revision number (D_XPM_PMP_PSE_REVISION)
   sPrinterSensor.dwTsLastUpdate  = 0;                       // Set timestamp of last update
   sPrinterSensor.dwDataValidFlag = 0;                       // Reset data valid flags (see D_XPM_PMP_PSE_DVF_...)

   //-- Check if data must be requested from the printer or if data is to be invalidated
   if (fInvalidateOnly) 
      goto DONE;

   //-- Inquire sensor data 
   {
      //-- Inquire sensor data 
      //.. Set read timeout printer response to inquiry to .. ms, depending if printer could be busy) 
      dwReadTimeOutMs = (!fPrinterMightBeBusy ? 300 : 15000);
      if (!Xpm_Pmp_Ifc_InquirePrinterData(psData, D_XPM_API_PID_TX_PRINTER_SENSOR, aubEcPrinterSensor, sizeof(aubEcPrinterSensor), aubPrinterSensor, sizeof(aubPrinterSensor), dwReadTimeOutMs)) 
         goto DONE;

      //-- Copy sensor data to storage
      memcpy(&sPrinterSensor.sPrinterSensor, &aubPrinterSensor[sizeof(TS_XPM_API_PCKT_HEADER)], sizeof(sPrinterSensor.sPrinterSensor));

       //-- Set sensor data valid flag
      sPrinterSensor.dwDataValidFlag |= D_XPM_PMP_PSE_DVF_PRINTER_SENSOR;
   }

   //-- Success (inquiry succesfull)
   fResult = true;

DONE:

   //-- Set time stamp
   sPrinterSensor.dwTsLastUpdate = GetTickCount();

   //-- Write status info to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_PRINTER_SENSOR, (PBYTE) &sPrinterSensor, sizeof(sPrinterSensor)))
      goto FAILURE;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
