/***********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmp_RegistryAccess.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Xpm-PrintMonPlugin)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Functions for registry access
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Xpm_Pmp_System.h"
#include "Xpm_PrintMonPlugin.h"
#include "Xpm_Pmp_Interface.h"
#include "Xpm_Pmp_RegistryAccess.h"

/*
** COMPILER
*/

/*
** MACROS 
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
//** FUNCTION     Xpm_Pmp_Ra_WritePrinterDataToRegistry
//** ABSTRACT     Writes data to registry for the current printer
//** PARAMETER    psData                Print monitor plugin data structure 
//**              pszValue              Key value in registry 
//**              pubInfo               Info structure which must be written
//**              dwSize                Size of info structure
//** RETURN       fResult               TRUE if success, FALSE otherwise
//******************************************************************************************

bool Xpm_Pmp_Ra_WritePrinterDataToRegistry
   (P_XPM_PLUGIN_DATA psData, PSTR pszValue, PBYTE pubInfo, DWORD dwSize)
{
   bool                            fResult            = false;   // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;             // Printer defaults structure
   HANDLE                          hPrinter           = NULL;    // Handle of printer
   
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID) || !pubInfo)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer
   //.. If defined, select printer that is coupled to the document
   //.. otherwise select printer for which the print monitor has been started 
   //.. Note that multiple printers may share the same port
   if (!OpenPrinter(psData->pszPrinterNameDoc ? psData->pszPrinterNameDoc : psData->pszPrinterName, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Write printer data to registry
   SetPrinterDataExA(hPrinter, D_XPM_PMP_REG_KEY, pszValue, REG_BINARY, pubInfo, dwSize);

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
//** FUNCTION     Xpm_Pmp_Ra_ReadPrinterDataFromRegistry
//** ABSTRACT     Reads data from registry for the current printer
//** PARAMETER    psData                Print monitor plugin data structure 
//**              pszValue              Key value in registry 
//**              pubInfo               Info structure which must be written
//**              dwSize                Size of info structure
//** RETURN       fResult               TRUE if success, FALSE otherwise
//******************************************************************************************

bool Xpm_Pmp_Ra_ReadPrinterDataFromRegistry
   (P_XPM_PLUGIN_DATA psData, PSTR pszValue, PBYTE pubInfo, DWORD dwSize)
{
   bool                            fResult            = false;   // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;             // Printer defaults structure
   HANDLE                          hPrinter           = NULL;    // Handle of printer
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID) || !pubInfo)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer 
   if (!OpenPrinter(psData->pszPrinterName, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Read data from registery and reset data if not (yet) available or if revision nr differs
   {
      DWORD dwType;   // Type of registry data
      DWORD dwNeeded; // Bytes needed
   
       if (    (GetPrinterDataExA(hPrinter, D_XPM_PMP_REG_KEY, pszValue, &dwType, pubInfo, dwSize, &dwNeeded) != ERROR_SUCCESS) 
            || (dwType != REG_BINARY)
          ) goto FAILURE;
   }

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
